import java.io.*;
import java.util.*;

import edu.uci.ics.jung.graph.*;

import net.deterlab.abac.Credential;
import net.deterlab.abac.CredentialGraph;
import net.deterlab.abac.Query;
import net.deterlab.abac.Role;
import net.deterlab.abac.Identity;

import java.security.KeyPair;


/**
 * Simple test of the native Java implementation of ABAC. Loads credentials
 * from an rt0 file and runs a query against them.
 */
public class GraphTest {
    public static void main(String[] args) throws IOException {
        if (args.length < 3) {
            System.out.println("Usage: GraphTest <files> <role> <principal>");
            System.out.println("    runs the query role <-?- principal and prints the result");
            System.exit(1);
        }

        CredentialGraph graph = new CredentialGraph();
	Map<String, Exception> errs = new HashMap<String, Exception>();

	for (int i= 0; i < args.length-2; i++) {
	    File f = new File(args[i]);

	    try {
		if (f.isDirectory()) 
		    for (Credential c :Credential.readDirectory(f, errs)) 
			graph.add_credential(c);
		else if (f.getPath().endsWith(".pem")) 
		    Credential.addIdentity(new Identity(f));
		else if (f.getPath().endsWith(".der"))
		    graph.add_credential(new Credential(f));
		else if (f.getPath().endsWith(".zip"))
		    for (Credential c :Credential.readZipFile(f, errs)) 
			graph.add_credential(c);
		else
		    System.out.println(f + " of unknown type");
	    }
	    catch (Exception e) {
		System.err.println("Failed to process " + f + ": " +e);
	    }
	}

	for (String f: errs.keySet()) System.err.println(f + " " + errs.get(f));

        //
        // run the query
        //

	Role role = new Role(args[args.length-2], true);
	Role prin = new Role(args[args.length-1], true);
        Query q = graph.querier();
        Graph<Role, Credential> ret = q.run(role.toString(), prin.toString());
	String fn = "attr";
	int n = 0;
	String suf = ".der";
        for (Credential c : ret.getEdges()) {
            System.out.println(c.simpleString());
	    if ( c.hasCertificate()) c.write(fn + n++ + suf);
	}

	fn = "id";
	n = 0;
	suf = ".pem";
	for (Identity i: Credential.identities()) {
	    System.out.println("ID: " + i);
	    i.write(fn + n++ + suf);
	}
	Credential.writeZipFile(graph.credentials(), 
		new File("./testout.zip"), true, true);
    }
}
